%efield
% Function which creates a 2D visulization of the electric field strength
% resulting from a set of point charges. Default output is a dipole field.
% Note the visualization is based upon the E field vectors in the x,y
% plane. A 3D field is actually calculated, and x,y,z coordinates of point
% charges are required as an input.
%
% Once a .png image of the field lines has been created, the magnitude of
% the field will be plotted via a coloured surface. Moving the mouse over
% the image will illustrate the x and y components of the electric field.
%
% [Ex,Ey,V,x,y] = efield( xq, yq, zq, q, xlimits, ylimits, plot_title, options )
%
% xq        Vector determining x coordinates of charges
% yq        Vector determining y coordinates of charges
% zq        Vector determining z coordinates of charges
% q         Vector determining charges /coulombs
% xlimits   [xmin,xmax] vector defining plot limits /m
% ylimits   [ymin,ymax] vector defining plot limits /m
% options   Structured array with plot options
%
% Ex        Matrix of electric field strength horizontal coordinates at (x,y,0)
% Ey        Matrix of electric field strength vertical  coordinates at (x,y,0)
% V         Matrix of the electric potential V(x,y,0)
% x         Matrix of x values
% y         Matrix of y values
%
% LAST UPDATED by Andy French Nov 2016.

function varargout = efield(varargin)

%Create global variable d in which to store parameters
global d

%Initialize draw E field vectors flag.
d.Efield = 0;

%Define grid size (there will be N^2 data points)
N = 70;

%If using field vectors, define how many per plot dimension
NN = 30;

%Fontsize for graphs
fsize = 16;

%

%Assign inputs
if nargin==0
    %Set default inputs
    a = 0.1;
    d.xq = a/2*[-1,1];
    d.yq = [0,0];
    d.zq = [0,0];
    d.q = 1.602e-19 *[-1,1];
    xlimits = a*[-1,1];
    ylimits = a*[-1,1];
    plot_title = 'Dipole';
    options.interp_background = 1;
    options.field_lines = 1;
    options.overlay_contour_lines = 0;
    options.plot_charges = 1;
else
    %Assume all inputs are fully specified
    d.xq = varargin{1};
    d.yq = varargin{2};
    d.zq = varargin{3};
    d.q = varargin{4};
    xlimits = varargin{5};
    ylimits = varargin{6};
    plot_title = varargin{7};
    options = varargin{8};
end

%

%Define x,y grid
xx = linspace( xlimits(1), xlimits(2), N );
yy = linspace( ylimits(1), ylimits(2), N );
[d.x,d.y] = meshgrid(xx,yy);

%Are field vectors chosen?
if options.field_lines == 0
    XX = linspace( xlimits(1), xlimits(2), NN );
    YY = linspace( ylimits(1), ylimits(2), NN );
    [X,Y] = meshgrid(XX,YY);
end

%Determine electric potential and field in x,y plane (i.e. z = 0 )
[d.Ex,d.Ey,d.V] = field_calc( d.x,d.y,d.xq,d.yq,d.zq, d.q );

%Determine log10 of electric field /Vm^-1 strength at all grid points
E = log10( sqrt( d.Ex.^2 + d.Ey.^2 ) );

%Determine typical value of E
d.Etyp = max( [ mean(mean(abs(d.Ex))),mean(mean(abs(d.Ey))) ] );

%Define outputs
if nargout~=0
    varargout{1} = d.Ex;
    varargout{2} = d.Ey;
    varargout{3} = d.V;
    varargout{4} = d.x;
    varargout{5} = d.y;
end

%If field vectors chosen, repeat for field vector points
if options.field_lines == 0
    
    %Determine electric potential and field in x,y direction
    [EX,EY,V] = field_calc( X,Y, d.xq,d.yq,d.zq, d.q );
    
    %Determine log10 of electric field /Vm^-1 strength at all grid points
    EE = log10( sqrt( EX.^2 + EY.^2 ) );
    
    %Determine appropriately log10 scaled quiver lines
    logEE = log10(EE) - min(min(log10(EE)));
    EX = (EX./EE).*logEE;
    EY = (EY./EE).*logEE;
end

%Plot electric field vectors as an overlay upon a colour-coded map of electric field
%strength

%Create new figure
d.fig = figure('color',[1 1 1],...
    'name','electric field' );

%Plot electric field strength as the background
if options.interp_background == 1;
    pcolor(d.x,d.y,E);
    shading interp
    alpha(0.3);
else
    contourf(d.x,d.y,E);
end
hold on;

%Set axis limits
axis equal;
xlim(xlimits);
ylim(ylimits);
axis manual

%Set colour scale
caxis([min(min(E)),max(max(E))]);
interp_colormap(N);
colorbar('fontsize',fsize);

%Overlay unfilled contour lines
if options.overlay_contour_lines == 1
    contour(d.x,d.y,E);
end

%Overlay field lines or field vectors
if options.field_lines == 0
    quiver(XX,YY,EX,EY,'color','w');
else
    s = streamslice(d.x,d.y,d.Ex,d.Ey,'color','w');
end

%Overlay positive and negative charges
if options.plot_charges==1
    negative = (d.q < 0);
    positive = (d.q >= 0);
    plot3(d.xq(negative),d.yq(negative),d.zq(negative),'b*','markersize',10)
    plot3(d.xq(positive),d.yq(positive),d.zq(positive),'r*','markersize',10)
end

%Grid lines and annotation
grid on;
set(gca,'fontsize',fsize);
xlabel('x /m','fontsize',fsize)
ylabel('y /m','fontsize',fsize)
title({plot_title,...
    'Colour scale is log_{10} of E field in Vm ^{-1}'},'fontsize',fsize)

%Save a .png file
print(d.fig,[plot_title,'.png'],'-dpng','-r300' );

%Update figure to include mouse movement and key press handlers
set( d.fig,'KeyPressFcn',@keypressfunc,...
    'WindowButtonDownFcn',@mouseclickdown,...
    'WindowButtonMotionFcn', @mousedrag,...
    'WindowButtonUpFcn',@mouseclickup);
d.xmin = xlimits(1);
d.xmax = xlimits(2);
d.ymin = ylimits(1);
d.ymax = ylimits(2);

%%

% Compute electric potential and electric field vectors
function [Ex,Ey,V] = field_calc( x,y, xq,yq,zq, q )

%Permittivity of free space / m^-3 kg^-1 s^4 A^2 (or Fm^-1)
e0 = 8.854e-12;

%Calculate electric potential and electric field in (x,y,z=0) plane
dim = size(x);
V = zeros(dim);
Ex = zeros(dim);
Ey = zeros(dim);
Xq = zeros( dim(1),dim(2),numel(q) );
Yq = zeros( dim(1),dim(2),numel(q) );
Zq = zeros( dim(1),dim(2),numel(q) );
Q = zeros( dim(1),dim(2),numel(q) );
for k=1:numel(q)
    Xq(:,:,k) = xq(k);
    Yq(:,:,k) = yq(k);
    Zq(:,:,k) = zq(k);
    Q(:,:,k) = q(k);
end
x = repmat(x,[1,1,numel(q)] );
y = repmat(y,[1,1,numel(q)] );
z = zeros( dim(1),dim(2),numel(q) );
r = sqrt( ( x - Xq ).^2 + ( y - Yq ).^2 + ( z - Zq ).^2 );
V  = sum(( Q./(4*pi*e0) )./r,3);
Ex = sum(( x - Xq ).*( Q./(4*pi*e0) )./(r.^3),3);
Ey = sum(( y - Yq ).*( Q./(4*pi*e0) )./(r.^3),3);

%%

%%%% KEYBOARD & MOUSE INTERACTION CALLBACKS %%%%

% e.g. key press handler, mouse click event manager,
% close request function etc

%Key press handler
function keypressfunc(hObject, eventdata, handles)
global d

%Get the string of the current key press
s = get( d.fig,'currentkey');

%Actions, depending on the key pressed...
if ( strcmp(s,'q') == 1 ) || ( strcmp(s,'Q') == 1 )
    %Quit GUI
    close(gcf);
else
    %Print screen
    drawnow;
    print( d.fig, ['efield ',strrep( datestr(now),':','-' ),'.png'],'-dpng','-r300' );
end

%%

%This function executes when the mouse is clicked
function mouseclickdown(hObject, eventdata, handles)
global d

%Store 'click point' coordinates (pixels from bottom left of the figure)
cp=get( gca,'currentpoint');
x0 = cp(1,1);
y0 = cp(1,2);

%Check 'click point' coordinates are within axes
xv = [ d.xmin, d.xmax, d.xmax, d.xmin ];
yv = [ d.ymin, d.ymin, d.ymax, d.ymax ];

%Determine x and y E field vectors
if ( inpolygon( x0, y0, xv, yv ) == 1 )
    
    %Enable Efield vector dynamic plotting
    d.Efield = 1;
    
    %Determine electric field strength at coordinates (x0,y0)
    [Ex,Ey,V] = field_calc( x0,y0, d.xq,d.yq,d.zq, d.q );
    
    %Normalize
    Ex = 0.2*( d.xmax - d.xmin )*Ex/d.Etyp ;
    Ey = 0.2*( d.xmax - d.xmin )*Ey/d.Etyp ;
    
    %Create field vectors
    d.Exarrow = plot( [x0, x0 + Ex], [y0,y0],'r','linewidth',2 );
    d.Eyarrow = plot( [x0, x0], [y0,y0 + Ey],'b','linewidth',2 );
    d.Earrow = plot( [x0, x0 + Ex], [y0,y0 + Ey],'g','linewidth',2 );
end

%%

%This function runs when a mouse is dragged following a click event, and
%before the mouse button is released
function mousedrag(hObject, eventdata, handles)
global d

%Store 'click point' coordinates (pixels from bottom left of the figure)
cp=get( gca,'currentpoint');
x0 = cp(1,1);
y0 = cp(1,2);

%Check 'click point' coordinates are within axes
xv = [ d.xmin, d.xmax, d.xmax, d.xmin ];
yv = [ d.ymin, d.ymin, d.ymax, d.ymax ];

%Overlay x and y components of E field
if ( inpolygon( x0, y0, xv, yv ) == 1 ) && ( d.Efield == 1 )
    
    %Determine electric field strength at coordinates (x0,y0)
    [Ex,Ey,V] = field_calc( x0,y0, d.xq,d.yq,d.zq, d.q );
    
    %Normalize
    Ex = 0.2*( d.xmax - d.xmin )*Ex/d.Etyp ;
    Ey = 0.2*( d.xmax - d.xmin )*Ey/d.Etyp ;
    
    %Update field vectors
    set( d.Exarrow, 'xdata',[x0, x0 + Ex],'ydata', [y0,y0] );
    set( d.Eyarrow, 'xdata',[x0, x0],'ydata', [y0,y0 + Ey] );
    set( d.Earrow, 'xdata',[x0, x0+ Ex],'ydata', [y0,y0 + Ey] );
end
drawnow;

%%

%This function executes when a mouse button is released
function mouseclickup(hObject, eventdata, handles)
global d

%Prevent any further dynamic update during mouse dragging
d.Efield = 0;

%Delete x an y field vectors
delete( d.Exarrow );
delete( d.Eyarrow );
delete( d.Earrow );

%%

%interp_colormap
% Function which interpolates current colormap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code